@@ -27,12 +27,12 @@
 #include <config.h>
 #include <string.h>
 
-#include "gdk-pixbuf-alias.h"
 #include "gdk-pixbuf-private.h"
 #include "gdk-pixbuf-animation.h"
 #include "gdk-pixbuf-io.h"
 #include "gdk-pixbuf-loader.h"
 #include "gdk-pixbuf-marshal.h"
+#include "gdk-pixbuf-alias.h"
 
 enum {
         SIZE_PREPARED,
@@ -52,7 +52,7 @@ static guint    pixbuf_loader_signals[LAST_SIGNAL] = { 0 };
 
 /* Internal data */
 
-#define LOADER_HEADER_SIZE 128
+#define LOADER_HEADER_SIZE 1024
 
 typedef struct
 {
@@ -485,7 +485,7 @@ gdk_pixbuf_loader_write (GdkPixbufLoader *loader,
         /* we expect it's not to be closed */
         g_return_val_if_fail (priv->closed == FALSE, FALSE);
   
-        if (priv->image_module == NULL)
+        if (count > 0 && priv->image_module == NULL)
                 {
                         gint eaten;
       
@@ -564,6 +564,7 @@ gdk_pixbuf_loader_new_with_type (const char *image_type,
         if (tmp != NULL)
                 {
                         g_propagate_error (error, tmp);
+                        gdk_pixbuf_loader_close (retval, NULL);
                         g_object_unref (retval);
                         return NULL;
                 }
@@ -619,10 +620,11 @@ gdk_pixbuf_loader_new_with_mime_type (const char *mime_type,
         retval = g_object_new (GDK_TYPE_PIXBUF_LOADER, NULL);
 
         tmp = NULL;
-        gdk_pixbuf_loader_load_module(retval, image_type, &tmp);
+        gdk_pixbuf_loader_load_module (retval, image_type, &tmp);
         if (tmp != NULL)
                 {
                         g_propagate_error (error, tmp);
+                        gdk_pixbuf_loader_close (retval, NULL);
                         g_object_unref (retval);
                         return NULL;
                 }
@@ -725,7 +727,9 @@ gdk_pixbuf_loader_close (GdkPixbufLoader *loader,
         /* we expect it's not closed */
         g_return_val_if_fail (priv->closed == FALSE, TRUE);
   
-        /* We have less the 128 bytes in the image.  Flush it, and keep going. */
+        /* We have less the LOADER_HEADER_SIZE bytes in the image.  
+         * Flush it, and keep going. 
+         */
         if (priv->image_module == NULL)
                 {
                         GError *tmp = NULL;
@@ -805,5 +809,6 @@ gdk_pixbuf_loader_get_format (GdkPixbufLoader *loader)
 }
 
 
-
+#define __GDK_PIXBUF_LOADER_C__
+#include "gdk-pixbuf-aliasdef.c"
 
