@@ -110,10 +110,15 @@ _gdk_windowing_init (void)
 
   _cf_rtf = RegisterClipboardFormat ("Rich Text Format");
   _cf_utf8_string = RegisterClipboardFormat ("UTF8_STRING");
+  _cf_image_bmp = RegisterClipboardFormat ("image/bmp");
 
-  _utf8_string = gdk_atom_intern ("UTF8_STRING", FALSE);
+  _gdk_selection_property = gdk_atom_intern ("GDK_SELECTION", FALSE);
+  _wm_transient_for = gdk_atom_intern ("WM_TRANSIENT_FOR", FALSE);
   _targets = gdk_atom_intern ("TARGETS", FALSE);
-
+  _save_targets = gdk_atom_intern ("SAVE_TARGETS", FALSE);
+  _utf8_string = gdk_atom_intern ("UTF8_STRING", FALSE);
+  _text = gdk_atom_intern ("TEXT", FALSE);
+  _compound_text = gdk_atom_intern ("COMPOUND_TEXT", FALSE);
   _text_uri_list = gdk_atom_intern ("text/uri-list", FALSE);
   _image_bmp = gdk_atom_intern ("image/bmp", FALSE);
 
@@ -121,10 +126,6 @@ _gdk_windowing_init (void)
   _gdk_win32_dropfiles = gdk_atom_intern ("DROPFILES_DND", FALSE);
   _gdk_ole2_dnd = gdk_atom_intern ("OLE2_DND", FALSE);
 
-  _gdk_selection_property = gdk_atom_intern ("GDK_SELECTION", FALSE);
-
-  _wm_transient_for = gdk_atom_intern ("WM_TRANSIENT_FOR", FALSE);
-
   _gdk_win32_selection_init ();
 }
 
@@ -186,13 +187,13 @@ gdk_screen_get_height (GdkScreen *screen)
 gint
 gdk_screen_get_width_mm (GdkScreen *screen)
 {
-  return (double) GetDeviceCaps (_gdk_display_hdc, HORZRES) / GetDeviceCaps (_gdk_display_hdc, LOGPIXELSX) * 25.4;
+  return (double) gdk_screen_get_width (screen) / GetDeviceCaps (_gdk_display_hdc, LOGPIXELSX) * 25.4;
 }
 
 gint
 gdk_screen_get_height_mm (GdkScreen *screen)
 {
-  return (double) GetDeviceCaps (_gdk_display_hdc, VERTRES) / GetDeviceCaps (_gdk_display_hdc, LOGPIXELSY) * 25.4;
+  return (double) gdk_screen_get_height (screen) / GetDeviceCaps (_gdk_display_hdc, LOGPIXELSY) * 25.4;
 }
 
 void
@@ -621,14 +622,15 @@ _gdk_win32_psstyle_to_string (DWORD pen_style)
   switch (pen_style & PS_STYLE_MASK)
     {
 #define CASE(x) case PS_##x: return #x
+      CASE (ALTERNATE);
+      CASE (SOLID);
       CASE (DASH);
+      CASE (DOT);
       CASE (DASHDOT);
       CASE (DASHDOTDOT);
-      CASE (DOT);
-      CASE (INSIDEFRAME);
       CASE (NULL);
-      CASE (SOLID);
       CASE (USERSTYLE);
+      CASE (INSIDEFRAME);
 #undef CASE
     default: return static_printf ("illegal_%d", pen_style & PS_STYLE_MASK);
     }
@@ -642,9 +644,9 @@ _gdk_win32_psendcap_to_string (DWORD pen_style)
   switch (pen_style & PS_ENDCAP_MASK)
     {
 #define CASE(x) case PS_ENDCAP_##x: return #x
-      CASE (FLAT);
       CASE (ROUND);
       CASE (SQUARE);
+      CASE (FLAT);
 #undef CASE
     default: return static_printf ("illegal_%d", pen_style & PS_ENDCAP_MASK);
     }
@@ -658,9 +660,9 @@ _gdk_win32_psjoin_to_string (DWORD pen_style)
   switch (pen_style & PS_JOIN_MASK)
     {
 #define CASE(x) case PS_JOIN_##x: return #x
+      CASE (ROUND);
       CASE (BEVEL);
       CASE (MITER);
-      CASE (ROUND);
 #undef CASE
     default: return static_printf ("illegal_%d", pen_style & PS_JOIN_MASK);
     }
@@ -931,7 +933,7 @@ _gdk_win32_cf_to_string (UINT format)
 
   switch (format)
     {
-#define CASE(x) case CF_##x: return #x
+#define CASE(x) case CF_##x: return "CF_" #x
       CASE (BITMAP);
       CASE (DIB);
 #ifdef CF_DIBV5
@@ -964,7 +966,7 @@ _gdk_win32_cf_to_string (UINT format)
 	  format <= CF_PRIVATELAST)
 	return static_printf ("CF_PRIVATE%d", format - CF_PRIVATEFIRST);
       if (GetClipboardFormatName (format, buf, sizeof (buf)))
-	return static_printf ("%s", buf);
+	return static_printf ("'%s'", buf);
       else
 	return static_printf ("unk-%#lx", format);
     }
