@@ -30,10 +30,7 @@
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkvbox.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
+G_BEGIN_DECLS
 
 #define GTK_TYPE_COLOR_SELECTION			(gtk_color_selection_get_type ())
 #define GTK_COLOR_SELECTION(obj)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_COLOR_SELECTION, GtkColorSelection))
@@ -88,14 +85,14 @@ void       gtk_color_selection_set_has_palette         (GtkColorSelection *color
 
 
 void     gtk_color_selection_set_current_color   (GtkColorSelection *colorsel,
-						  GdkColor          *color);
+						  const GdkColor    *color);
 void     gtk_color_selection_set_current_alpha   (GtkColorSelection *colorsel,
 						  guint16            alpha);
 void     gtk_color_selection_get_current_color   (GtkColorSelection *colorsel,
 						  GdkColor          *color);
 guint16  gtk_color_selection_get_current_alpha   (GtkColorSelection *colorsel);
 void     gtk_color_selection_set_previous_color  (GtkColorSelection *colorsel,
-						  GdkColor          *color);
+						  const GdkColor    *color);
 void     gtk_color_selection_set_previous_alpha  (GtkColorSelection *colorsel,
 						  guint16            alpha);
 void     gtk_color_selection_get_previous_color  (GtkColorSelection *colorsel,
@@ -128,9 +125,6 @@ void gtk_color_selection_set_update_policy (GtkColorSelection *colorsel,
 					    GtkUpdateType      policy);
 #endif /* GTK_DISABLE_DEPRECATED */
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
-
+G_END_DECLS
 
 #endif /* __GTK_COLOR_SELECTION_H__ */
