@@ -25,7 +25,6 @@
  */
 
 #include <config.h>
-#include "gtkalias.h"
 #include "gtkbutton.h"
 #include "gtkdialog.h"
 #include "gtkhbbox.h"
@@ -37,6 +36,8 @@
 #include "gtkmain.h"
 #include "gtkintl.h"
 #include "gtkbindings.h"
+#include "gtkprivate.h"
+#include "gtkalias.h"
 
 #define GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_DIALOG, GtkDialogPrivate))
 
@@ -144,11 +145,11 @@ gtk_dialog_class_init (GtkDialogClass *class)
 
   g_object_class_install_property (gobject_class,
                                    PROP_HAS_SEPARATOR,
-                                   g_param_spec_boolean ("has_separator",
+                                   g_param_spec_boolean ("has-separator",
 							 P_("Has separator"),
 							 P_("The dialog has a separator bar above its buttons"),
                                                          TRUE,
-                                                         G_PARAM_READWRITE));
+                                                         GTK_PARAM_READWRITE));
   
   dialog_signals[RESPONSE] =
     g_signal_new ("response",
@@ -170,30 +171,30 @@ gtk_dialog_class_init (GtkDialogClass *class)
 		  G_TYPE_NONE, 0);
   
   gtk_widget_class_install_style_property (widget_class,
-					   g_param_spec_int ("content_area_border",
+					   g_param_spec_int ("content-area-border",
                                                              P_("Content area border"),
                                                              P_("Width of border around the main dialog area"),
                                                              0,
                                                              G_MAXINT,
                                                              2,
-                                                             G_PARAM_READABLE));
+                                                             GTK_PARAM_READABLE));
   gtk_widget_class_install_style_property (widget_class,
-                                           g_param_spec_int ("button_spacing",
+                                           g_param_spec_int ("button-spacing",
                                                              P_("Button spacing"),
                                                              P_("Spacing between buttons"),
                                                              0,
                                                              G_MAXINT,
                                                              10,
-                                                             G_PARAM_READABLE));
+                                                             GTK_PARAM_READABLE));
   
   gtk_widget_class_install_style_property (widget_class,
-                                           g_param_spec_int ("action_area_border",
+                                           g_param_spec_int ("action-area-border",
                                                              P_("Action area border"),
                                                              P_("Width of border around the button area at the bottom of the dialog"),
                                                              0,
                                                              G_MAXINT,
                                                              5,
-                                                             G_PARAM_READABLE));
+                                                             GTK_PARAM_READABLE));
 
   binding_set = gtk_binding_set_by_class (class);
   
@@ -212,12 +213,9 @@ update_spacings (GtkDialog *dialog)
   widget = GTK_WIDGET (dialog);
 
   gtk_widget_style_get (widget,
-                        "content_area_border",
-                        &content_area_border,
-                        "button_spacing",
-                        &button_spacing,
-                        "action_area_border",
-                        &action_area_border,
+                        "content-area-border", &content_area_border,
+                        "button-spacing", &button_spacing,
+                        "action-area-border", &action_area_border,
                         NULL);
 
   gtk_container_set_border_width (GTK_CONTAINER (dialog->vbox),
@@ -345,23 +343,29 @@ gtk_dialog_map (GtkWidget *widget)
   if (!window->focus_widget)
     {
       GList *children, *tmp_list;
+      GtkWidget *first_focus = NULL;
       
       do 
 	{
 	  g_signal_emit_by_name (window, "move_focus", GTK_DIR_TAB_FORWARD);
+
+	  if (first_focus == NULL)
+	    first_focus = window->focus_widget;
+	  else if (first_focus == window->focus_widget)
+	    break;
 	}
       while (GTK_IS_LABEL (window->focus_widget));
 
       tmp_list = children = gtk_container_get_children (GTK_CONTAINER (dialog->action_area));
-
+      
       while (tmp_list)
 	{
 	  GtkWidget *child = tmp_list->data;
-
-	  if (window->focus_widget == NULL ||
-	      (child == window->focus_widget && 
-	       child != window->default_widget && 
-	       window->default_widget))
+	  
+	  if ((window->focus_widget == NULL || 
+	       child == window->focus_widget) && 
+	      child != window->default_widget &&
+	      window->default_widget)
 	    {
 	      gtk_widget_grab_focus (window->default_widget);
 	      break;
@@ -369,7 +373,7 @@ gtk_dialog_map (GtkWidget *widget)
 	  
 	  tmp_list = tmp_list->next;
 	}
-
+      
       g_list_free (children);
     }
 }
@@ -546,18 +550,9 @@ get_response_data (GtkWidget *widget,
 static void
 action_widget_activated (GtkWidget *widget, GtkDialog *dialog)
 {
-  ResponseData *ad;
   gint response_id;
   
-  g_return_if_fail (GTK_IS_DIALOG (dialog));
-
-  response_id = GTK_RESPONSE_NONE;
-  
-  ad = get_response_data (widget, TRUE);
-
-  g_assert (ad != NULL);
-  
-  response_id = ad->response_id;
+  response_id = gtk_dialog_get_response_for_widget (dialog, widget);
 
   gtk_dialog_response (dialog, response_id);
 }
@@ -581,7 +576,7 @@ gtk_dialog_add_action_widget (GtkDialog *dialog,
                               gint       response_id)
 {
   ResponseData *ad;
-  gint signal_id = 0;
+  guint signal_id;
   
   g_return_if_fail (GTK_IS_DIALOG (dialog));
   g_return_if_fail (GTK_IS_WIDGET (child));
@@ -593,7 +588,7 @@ gtk_dialog_add_action_widget (GtkDialog *dialog,
   if (GTK_IS_BUTTON (child))
     signal_id = g_signal_lookup ("clicked", GTK_TYPE_BUTTON);
   else
-    signal_id = GTK_WIDGET_GET_CLASS (child)->activate_signal != 0;
+    signal_id = GTK_WIDGET_GET_CLASS (child)->activate_signal;
 
   if (signal_id)
     {
@@ -735,8 +730,7 @@ gtk_dialog_set_response_sensitive (GtkDialog *dialog,
   while (tmp_list != NULL)
     {
       GtkWidget *widget = tmp_list->data;
-      ResponseData *rd = g_object_get_data (G_OBJECT (widget),
-                                            "gtk-dialog-response-data");
+      ResponseData *rd = get_response_data (widget, FALSE);
 
       if (rd && rd->response_id == response_id)
         gtk_widget_set_sensitive (widget, setting);
@@ -771,8 +765,7 @@ gtk_dialog_set_default_response (GtkDialog *dialog,
   while (tmp_list != NULL)
     {
       GtkWidget *widget = tmp_list->data;
-      ResponseData *rd = g_object_get_data (G_OBJECT (widget),
-                                            "gtk-dialog-response-data");
+      ResponseData *rd = get_response_data (widget, FALSE);
 
       if (rd && rd->response_id == response_id)
 	gtk_widget_grab_default (widget);
@@ -827,7 +820,7 @@ gtk_dialog_set_has_separator (GtkDialog *dialog,
       dialog->separator = NULL;
     }
 
-  g_object_notify (G_OBJECT (dialog), "has_separator");
+  g_object_notify (G_OBJECT (dialog), "has-separator");
 }
 
 /**
@@ -969,16 +962,16 @@ run_destroy_handler (GtkDialog *dialog, gpointer data)
  * 
  * Note that even though the recursive main loop gives the effect of a
  * modal dialog (it prevents the user from interacting with other 
- * windows while the dialog is run), callbacks such as timeouts, 
- * IO channel watches, DND drops, etc, <emphasis>will</emphasis> be 
- * triggered during a gtk_dialog_run() call.
+ * windows in the same window group while the dialog is run), callbacks 
+ * such as timeouts, IO channel watches, DND drops, etc, <emphasis>will</emphasis> 
+ * be triggered during a gtk_dialog_run() call.
  * 
  * Return value: response ID
  **/
 gint
 gtk_dialog_run (GtkDialog *dialog)
 {
-  RunInfo ri = { NULL, GTK_RESPONSE_NONE, NULL };
+  RunInfo ri = { NULL, GTK_RESPONSE_NONE, NULL, FALSE };
   gboolean was_modal;
   gulong response_handler;
   gulong unmap_handler;
@@ -1029,7 +1022,6 @@ gtk_dialog_run (GtkDialog *dialog)
   g_main_loop_unref (ri.loop);
 
   ri.loop = NULL;
-  ri.destroyed = FALSE;
   
   if (!ri.destroyed)
     {
@@ -1057,9 +1049,22 @@ _gtk_dialog_set_ignore_separator (GtkDialog *dialog,
   priv->ignore_separator = ignore_separator;
 }
 
+/**
+ * gtk_dialog_get_response_for_widget:
+ * @dialog: a #GtkDialog
+ * @widget: a widget in the action area of @dialog
+ *
+ * Gets the response id of a widget in the action area
+ * of a dialog.
+ *
+ * Returns: the response id of @widget, or %GTK_RESPONSE_NONE
+ *  if @widget doesn't have a response id set.
+ *
+ * Since: 2.8
+ */
 gint
-_gtk_dialog_get_response_for_widget (GtkDialog *dialog,
-				     GtkWidget *widget)
+gtk_dialog_get_response_for_widget (GtkDialog *dialog,
+				    GtkWidget *widget)
 {
   ResponseData *rd;
 
@@ -1231,3 +1236,5 @@ gtk_dialog_set_alternative_button_order_from_array (GtkDialog *dialog,
     }
 }
 
+#define __GTK_DIALOG_C__
+#include "gtkaliasdef.c"
