@@ -22,13 +22,13 @@
 #include <string.h>
 #include <locale.h>
 
-#include "gtkalias.h"
 #include "gtkimmulticontext.h"
 #include "gtkimmodule.h"
 #include "gtkmain.h"
 #include "gtkradiomenuitem.h"
 #include "gtkintl.h"
 #include "gtkprivate.h"
+#include "gtkalias.h"
 
 struct _GtkIMMulticontextPrivate
 {
@@ -567,3 +567,5 @@ gtk_im_multicontext_append_menuitems (GtkIMMulticontext *context,
   g_free (contexts);
 }
 
+#define __GTK_IM_MULTICONTEXT_C__
+#include "gtkaliasdef.c"
