@@ -127,6 +127,7 @@ get_slider_button (GtkPathBar  *path_bar,
   gtk_widget_push_composite_child ();
 
   button = gtk_button_new ();
+  gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
   gtk_container_add (GTK_CONTAINER (button), gtk_arrow_new (arrow_type, GTK_SHADOW_OUT));
   gtk_container_add (GTK_CONTAINER (path_bar), button);
   gtk_widget_show_all (button);
@@ -192,8 +193,9 @@ gtk_path_bar_class_init (GtkPathBarClass *path_bar_class)
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkPathBarClass, path_clicked),
 		  NULL, NULL,
-		  _gtk_marshal_VOID__POINTER_BOOLEAN,
-		  G_TYPE_NONE, 2,
+		  _gtk_marshal_VOID__POINTER_POINTER_BOOLEAN,
+		  G_TYPE_NONE, 3,
+		  G_TYPE_POINTER,
 		  G_TYPE_POINTER,
 		  G_TYPE_BOOLEAN);
 }
@@ -307,13 +309,19 @@ gtk_path_bar_update_slider_buttons (GtkPathBar *path_bar)
 
       button = BUTTON_DATA (path_bar->button_list->data)->button;
       if (gtk_widget_get_child_visible (button))
-	gtk_widget_set_sensitive (path_bar->down_slider_button, FALSE);
+	{
+	  gtk_path_bar_stop_scrolling (path_bar);
+	  gtk_widget_set_sensitive (path_bar->down_slider_button, FALSE);
+	}
       else
 	gtk_widget_set_sensitive (path_bar->down_slider_button, TRUE);
 
       button = BUTTON_DATA (g_list_last (path_bar->button_list)->data)->button;
       if (gtk_widget_get_child_visible (button))
-	gtk_widget_set_sensitive (path_bar->up_slider_button, FALSE);
+	{
+	  gtk_path_bar_stop_scrolling (path_bar);
+	  gtk_widget_set_sensitive (path_bar->up_slider_button, FALSE);
+	}
       else
 	gtk_widget_set_sensitive (path_bar->up_slider_button, TRUE);
     }
@@ -732,9 +740,9 @@ gtk_path_bar_scroll_timeout (GtkPathBar *path_bar)
 
   if (path_bar->timer)
     {
-      if (GTK_WIDGET_HAS_FOCUS (path_bar->up_slider_button))
+      if (path_bar->scrolling_up)
 	gtk_path_bar_scroll_up (path_bar->up_slider_button, path_bar);
-      else if (GTK_WIDGET_HAS_FOCUS (path_bar->down_slider_button))
+      else if (path_bar->scrolling_down)
 	gtk_path_bar_scroll_down (path_bar->down_slider_button, path_bar);
 
       if (path_bar->need_timer) 
@@ -772,18 +780,23 @@ gtk_path_bar_slider_button_press (GtkWidget      *widget,
 				  GdkEventButton *event,
 				  GtkPathBar     *path_bar)
 {
-  if (!GTK_WIDGET_HAS_FOCUS (widget))
-    gtk_widget_grab_focus (widget);
-
   if (event->type != GDK_BUTTON_PRESS || event->button != 1)
     return FALSE;
 
   path_bar->ignore_click = FALSE;
 
   if (widget == path_bar->up_slider_button)
-    gtk_path_bar_scroll_up (path_bar->up_slider_button, path_bar);
-  else if (widget == path_bar->down_slider_button)
-     gtk_path_bar_scroll_down (path_bar->down_slider_button, path_bar);
+    {
+      path_bar->scrolling_down = FALSE;
+      path_bar->scrolling_up = TRUE;
+      gtk_path_bar_scroll_up (path_bar->up_slider_button, path_bar);
+    }
+  else if (widget == path_bar->down_slider_button) 
+    {
+      path_bar->scrolling_up = FALSE;
+      path_bar->scrolling_down = TRUE;
+      gtk_path_bar_scroll_down (path_bar->down_slider_button, path_bar);
+    }
 
   if (!path_bar->timer)
     {
@@ -928,6 +941,7 @@ button_clicked_cb (GtkWidget *button,
   GtkPathBar *path_bar;
   GList *button_list;
   gboolean child_is_hidden;
+  GtkFilePath *child_path;
 
   button_data = BUTTON_DATA (data);
   if (button_data->ignore_changes)
@@ -945,12 +959,17 @@ button_clicked_cb (GtkWidget *button,
       ButtonData *child_data;
 
       child_data = BUTTON_DATA (button_list->prev->data);
+      child_path = child_data->path;
       child_is_hidden = child_data->file_is_hidden;
     }
   else
-    child_is_hidden = FALSE;
+    {
+      child_path = NULL;
+      child_is_hidden = FALSE;
+    }
 
-  g_signal_emit (path_bar, path_bar_signals [PATH_CLICKED], 0, button_data->path, child_is_hidden);
+  g_signal_emit (path_bar, path_bar_signals [PATH_CLICKED], 0,
+		 button_data->path, child_path, child_is_hidden);
 }
 
 static GdkPixbuf *
@@ -1016,13 +1035,7 @@ button_data_free (ButtonData *button_data)
 static const char *
 get_dir_name (ButtonData *button_data)
 {
-  if (button_data->type == HOME_BUTTON)
-    return _("Home");
-  else if (button_data->type == DESKTOP_BUTTON)
-    return _("Desktop");
-  else
-    return button_data->dir_name;
-
+  return button_data->dir_name;
 }
 
 /* We always want to request the same size for the label, whether
@@ -1129,7 +1142,7 @@ button_drag_data_get_cb (GtkWidget          *widget,
   gtk_selection_data_set (selection_data,
 			  selection_data->target,
 			  8,
-			  uri_list,
+			  (guchar *)uri_list,
 			  strlen (uri_list));
   g_free (uri_list);
 }
@@ -1155,6 +1168,7 @@ make_directory_button (GtkPathBar  *path_bar,
 
   button_data->type = find_button_type (path_bar, path);
   button_data->button = gtk_toggle_button_new ();
+  gtk_button_set_focus_on_click (GTK_BUTTON (button_data->button), FALSE);
 
   switch (button_data->type)
     {
