@@ -31,6 +31,7 @@
 
 #include "gtktypeutils.h"
 #include "gtkobject.h"
+#include "gtkalias.h"
 
 
 /* --- functions --- */
@@ -197,3 +198,6 @@ gtk_type_flags_find_value (GtkType      flags_type,
   return value;
 }
 
+
+#define __GTK_TYPE_UTILS_C__
+#include "gtkaliasdef.c"
